<?php
namespace Bricks;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * Responsible for handling the custom redirection logic for authentication-related pages.
 *
 * Login page
 * Registration page
 * Lost password page
 * Reset password page
 *
 * @since 1.9.2
 */
class Auth_Redirects {
	public function __construct() {
		add_action( 'wp_loaded', [ $this, 'handle_auth_redirects' ] );
	}

	/**
	 * Main function to handle authentication redirects.
	 * Depending on the current URL and the action parameter, decides which page to redirect to.
	 */
	public function handle_auth_redirects() {
		$request_uri      = esc_url_raw( $_SERVER['REQUEST_URI'] );
		$current_url_path = parse_url( home_url( $request_uri ), PHP_URL_PATH );

		$wp_login_url_path         = parse_url( wp_login_url(), PHP_URL_PATH );
		$wp_registration_url_path  = parse_url( wp_registration_url(), PHP_URL_PATH );
		$wp_lost_password_url_path = parse_url( wp_lostpassword_url(), PHP_URL_PATH );

		$action = isset( $_GET['action'] ) ? sanitize_key( $_GET['action'] ) : null;

		if ( $current_url_path == $wp_login_url_path ) { // Login page & actions
			switch ( $action ) {
				case null:
					$this->redirect_to_custom_login_page();
					break;
				case 'lostpassword':
					$this->redirect_to_custom_lost_password_page();
					break;
				case 'register':
					$this->redirect_to_custom_registration_page();
					break;
				case 'rp': // Reset password
					$this->redirect_to_custom_reset_password_page();
					break;
			}
		} elseif ( $current_url_path == $wp_registration_url_path ) { // Registration page fallback
			$this->redirect_to_custom_registration_page();
		} elseif ( $current_url_path == $wp_lost_password_url_path ) { // Lost password page fallback
			$this->redirect_to_custom_lost_password_page();
		}
	}

	/**
	 * Redirects to the custom login page if it's set and valid.
	 */
	private function redirect_to_custom_login_page() {
		$selected_login_page_id = Database::get_setting( 'login_page' );
		$this->redirect_if_valid_page( $selected_login_page_id );
	}

	/**
	 * Redirects to the custom lost password page if it's set and valid.
	 */
	private function redirect_to_custom_lost_password_page() {
		$selected_lost_password_page_id = Database::get_setting( 'lost_password_page' );
		$this->redirect_if_valid_page( $selected_lost_password_page_id );
	}

	/**
	 * Redirects to the custom registration page if it's set and valid.
	 */
	private function redirect_to_custom_registration_page() {
		$selected_registration_page_id = Database::get_setting( 'registration_page' );
		$this->redirect_if_valid_page( $selected_registration_page_id );
	}

	/**
	 * Redirects to the custom reset password page if it's set and valid.
	 */
	private function redirect_to_custom_reset_password_page() {
		$selected_reset_password_page_id = Database::get_setting( 'reset_password_page' );
		$this->redirect_if_valid_page( $selected_reset_password_page_id );
	}

	/**
	 * Helper function to redirect to the provided page if it's valid.
	 * If the page is not valid, redirects to a default URL if provided.
	 *
	 * @param int         $selected_page_id The ID of the page to redirect to.
	 * @param string|null $default_url The default URL to redirect to if the page is not valid.
	 */
	private function redirect_if_valid_page( $selected_page_id ) {
		if ( $this->is_custom_page_valid( $selected_page_id ) ) {
			$custom_url = get_permalink( $selected_page_id );

			// Preserve query parameters
			if ( ! empty( $_SERVER['QUERY_STRING'] ) ) {
				$custom_url = add_query_arg( $_GET, $custom_url );
			}

			if ( $custom_url ) {
				wp_safe_redirect( $custom_url );
				exit;
			}
		}
	}

	/**
	 * Checks if the custom page is valid.
	 *
	 * @param int $page_id
	 *
	 * @return bool
	 */
	private function is_custom_page_valid( $page_id ) {
		return $page_id && get_post_status( $page_id ) === 'publish';
	}
}
